<?php

namespace Drupal\css_js_performance_improvement\Service;

use Drupal\Core\File\FileSystemInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class DefaultService.
 */
final class DefaultService {

  /**
   * @var FileSystemInterface $fileSystem
   */
  protected $fileSystem;

  /**
   * {@inheritdoc}
   */
  public function __construct(FileSystemInterface $file_system) {
    $this->fileSystem = $file_system;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('file_system'),
    );
  }

  /**
   * 
   * CSS Minifier.
   * 
   * Info:
   * https://gist.github.com/Rodrigo54/93169db48194d470188f
   */
  public function minify_css($input) {
    if(trim($input) === "") return $input;
    return preg_replace(
        array(
            // Remove comment(s)
            '#("(?:[^"\\\]++|\\\.)*+"|\'(?:[^\'\\\\]++|\\\.)*+\')|\/\*(?!\!)(?>.*?\*\/)|^\s*|\s*$#s',
            // Remove unused white-space(s)
            '#("(?:[^"\\\]++|\\\.)*+"|\'(?:[^\'\\\\]++|\\\.)*+\'|\/\*(?>.*?\*\/))|\s*+;\s*+(})\s*+|\s*+([*$~^|]?+=|[{};,>~]|\s(?![0-9\.])|!important\b)\s*+|([[(:])\s++|\s++([])])|\s++(:)\s*+(?!(?>[^{}"\']++|"(?:[^"\\\]++|\\\.)*+"|\'(?:[^\'\\\\]++|\\\.)*+\')*+{)|^\s++|\s++\z|(\s)\s+#si',
            // Replace `0(cm|em|ex|in|mm|pc|pt|px|vh|vw|%)` with `0`
            '#(?<=[\s:])(0)(cm|em|ex|in|mm|pc|pt|px|vh|vw|%)#si',
            // Replace `:0 0 0 0` with `:0`
            '#:(0\s+0|0\s+0\s+0\s+0)(?=[;\}]|\!important)#i',
            // Replace `background-position:0` with `background-position:0 0`
            '#(background-position):0(?=[;\}])#si',
            // Replace `0.6` with `.6`, but only when preceded by `:`, `,`, `-` or a white-space
            '#(?<=[\s:,\-])0+\.(\d+)#s',
            // Minify string value
            '#(\/\*(?>.*?\*\/))|(?<!content\:)([\'"])([a-z_][a-z0-9\-_]*?)\2(?=[\s\{\}\];,])#si',
            '#(\/\*(?>.*?\*\/))|(\burl\()([\'"])([^\s]+?)\3(\))#si',
            // Minify HEX color code
            '#(?<=[\s:,\-]\#)([a-f0-6]+)\1([a-f0-6]+)\2([a-f0-6]+)\3#i',
            // Replace `(border|outline):none` with `(border|outline):0`
            '#(?<=[\{;])(border|outline):none(?=[;\}\!])#',
            // Remove empty selector(s)
            '#(\/\*(?>.*?\*\/))|(^|[\{\}])(?:[^\s\{\}]+)\{\}#s'
        ),
        array(
            '$1',
            '$1$2$3$4$5$6$7',
            '$1',
            ':0',
            '$1:0 0',
            '.$1',
            '$1$3',
            '$1$2$4$5',
            '$1$2$3',
            '$1:0',
            '$1$2'
        ),
    $input);
  }

  /**
   * 
   * JS Minifier.
   * 
   * Info:
   * https://gist.github.com/Rodrigo54/93169db48194d470188f
   */
  function minify_js($input) {
    if(trim($input) === "") return $input;
    return preg_replace(
        array(
            // Remove comment(s)
            '#\s*("(?:[^"\\\]++|\\\.)*+"|\'(?:[^\'\\\\]++|\\\.)*+\')\s*|\s*\/\*(?!\!|@cc_on)(?>[\s\S]*?\*\/)\s*|\s*(?<![\:\=])\/\/.*(?=[\n\r]|$)|^\s*|\s*$#',
            // Remove white-space(s) outside the string and regex
            '#("(?:[^"\\\]++|\\\.)*+"|\'(?:[^\'\\\\]++|\\\.)*+\'|\/\*(?>.*?\*\/)|\/(?!\/)[^\n\r]*?\/(?=[\s.,;]|[gimuy]|$))|\s*([!%&*\(\)\-=+\[\]\{\}|;:,.<>?\/])\s*#s',
            // Remove the last semicolon
            '#;+\}#',
            // Minify object attribute(s) except JSON attribute(s). From `{'foo':'bar'}` to `{foo:'bar'}`
            '#([\{,])([\'])(\d+|[a-z_][a-z0-9_]*)\2(?=\:)#i',
            // --ibid. From `foo['bar']` to `foo.bar`
            '#([a-z0-9_\)\]])\[([\'"])([a-z_][a-z0-9_]*)\2\]#i'
        ),
        array(
            '$1',
            '$1$2',
            '}',
            '$1$3',
            '$1.$3'
        ),
    $input);
  }

  public function generate_aggregated_css($css_files, $file_name) {
    $file_path = 'public://public/css_js_performance_improvement/css/';
    file_put_contents($file_path . $file_name, '.dummy{min-width:0;min-height:0;}');
    $file_content = '/* Empty content, at the moment... */';
    
    foreach($css_files as $name => $data) {
      if($name !== $file_path . $file_name) {
        @$file_content .= file_get_contents($this->fileSystem->realpath($css_files[$name]['data']));
        @file_put_contents($file_path . $file_name, $file_content);
      }
    }

    file_put_contents(
      $file_path . $file_name, 
      $this->minify_css(
        file_get_contents(
          $this->fileSystem->realpath($file_path . $file_name)
        )
      )
    );
  }

  public function generate_aggregated_js($js_files, $file_name) {
    $file_path = 'public://public/css_js_performance_improvement/js/';
    file_put_contents($file_path . $file_name, '/* Empty content, at the moment... */');
    $file_content = '/* Empty content, at the moment... */';
    
    foreach($js_files as $name => $data) {
      if($name !== $file_path . $file_name) {
        @$file_content .= file_get_contents($this->fileSystem->realpath($js_files[$name]['data']));
        @file_put_contents($file_path . $file_name, $file_content);
      }
    }

    file_put_contents(
      $file_path . $file_name, 
      $this->minify_js(
        file_get_contents(
          $this->fileSystem->realpath($file_path . $file_name)
        )
      )
    );
  }

  public function remove_all_folder_files($file_path) {
    $files = glob($file_path);
    $operation_status = true;
    foreach($files as $file) { 
      if(!unlink($file)) {
        $operation_status = false;
      }
    } return $operation_status; 
  }

}
